"""Module for parsing Python source files and extracting structured information using AST.

Copyright (c) 2024 Neil Schneider
"""

import ast
from pathlib import Path
from typing import ClassVar

from slimcontext.parsers.baseparser import BaseFileParser
from slimcontext.parsers.pyparse.assignments import PyAssignment
from slimcontext.parsers.pyparse.classes import PyClass
from slimcontext.parsers.pyparse.functions import PyFunction
from slimcontext.parsers.pyparse.imports import PyImport
from slimcontext.parsers.pyparse.pybase import PyNodeBase
from slimcontext.utils.logger import setup_logger

logger = setup_logger(__name__)


class PyParser(BaseFileParser):
    """Parses Python source files and extracts structured information using AST."""

    extensions: ClassVar[list[str]] = ['.py']

    def __init__(self, root_dir: Path | None = None) -> None:
        """Initialize the PyParser instance.

        Args:
            root_dir (Path, optional): The git repo's root directory. Defaults to None.
        """
        super().__init__(extensions=self.extensions, root_dir=root_dir)

    def context_body(self, source: str, file_path: Path) -> list[str]:
        """Extract comprehensive information from a Python source file using AST.

        Args:
            source (str): The raw text of the source file.
            file_path (Path): The path to the Python source file.

        Returns:
            list[str]: A list of all the context from the module.
        """
        try:
            tree = ast.parse(source=source, filename=str(file_path))
        except SyntaxError as e:
            logger.warning('Syntax error in %s: %s', file_path, e)
            tree = None

        all_nodes: list = (
            [self._process_node(node=node, source=source) for node in ast.iter_child_nodes(tree)]
            if tree
            else []
        )
        processed_nodes: list[PyNodeBase] = [node for node in all_nodes if node]

        body: list[str] = []
        module_docstring: str | None = ast.get_docstring(tree) if tree else None
        if module_docstring:
            body.extend([module_docstring])
        for node in processed_nodes:
            body.extend(node.context)

        return body

    @staticmethod
    def _process_node(node: ast.AST, source: str) -> PyNodeBase | None:
        """Process a single AST node and extract relevant information.

        Args:
            node (ast.AST): The AST node to process.
            source (str): The raw text of the source file.

        Returns:
            One of the PyNode objects built from PyNodeBase.
        """
        if isinstance(node, ast.Import | ast.ImportFrom):
            return PyImport(node=node, source=source)
        if isinstance(node, ast.Assign):
            return PyAssignment(node=node, source=source)
        if isinstance(node, ast.FunctionDef | ast.AsyncFunctionDef):
            return PyFunction(node=node, source=source)
        if isinstance(node, ast.ClassDef):
            return PyClass(node=node, source=source)
        logger.debug('Found uncategorized node type: %s', type(node))
        return None

    def generate_slim_context(self, source: str, file_path: Path) -> str:
        """Generate a slim textual context from the repository information for the LLM.

        Args:
            source (str): The raw text of the source file.
            file_path (Path): The path to the Python source file.

        Returns:
            str: A string representation of the repository context.
        """
        context_lines: list[str] = []
        context_lines.extend(self.context_header(file_path=file_path))
        context_lines.extend(self.context_body(source=source, file_path=file_path))

        logger.debug('Generated slim context for LLM.')
        return '\n'.join(context_lines)

    def generate_full_context(self, source: str, file_path: Path) -> str:
        """Return the entire content of a Python source file.

        Args:
            source (str): The raw text of the source file.
            file_path (Path): The path to the Python source file.

        Returns:
            str: Full text content of the file.
        """
        context_lines: list[str] = []
        context_lines.extend(self.context_header(file_path=file_path))
        context_lines.append(source)

        return '\n'.join(context_lines)


if __name__ == '__main__':

    def example_use() -> None:
        """Small example of how to use PyParser."""
        logger_ex = setup_logger(f'Example use of {__name__}')
        parser = PyParser()
        # Sample file path for demonstration purposes
        file_path = Path('slimcontext/parsers/pyparse/pyparser.py')
        context = parser.generate_context(file_path)
        logger_ex.info('Example context generated by PyParser: %s', context)
